/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Project, NZDL, University of Waikato
 *
 * Copyright (C) 2003 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.collection.BasicCollectionConfiguration;
import org.greenstone.gatherer.gui.GLIButton;
import org.greenstone.gatherer.gui.SimpleMenuBar;
import org.greenstone.gatherer.gui.ModalDialog;
import org.greenstone.gatherer.util.StaticStrings;
import org.greenstone.gatherer.util.Utility;

public class NewCollectionDetailsPrompt
    extends ModalDialog {

    static public boolean titleClashes(String title, File current_config_file) {
	// An empty collection title never clashes with anything. I may look ugly in the final collection but there is nothing wrong with having no title for a particular language.
	if(title == null || title.length() == 0) {
	    return false;
	}
	File collection_directory = new File(Utility.getCollectionDir(Gatherer.config.gsdl_path));
	File children[] = collection_directory.listFiles();
	for(int i = 0; children != null && i < children.length; i++) {
	    if(children[i].isDirectory()) {
		File config_file = new File(children[i], Utility.CONFIG_DIR);
		if(current_config_file == null || !config_file.equals(current_config_file)) {
		    BasicCollectionConfiguration other_collection = new BasicCollectionConfiguration(config_file);
		    if(other_collection.getName().equalsIgnoreCase(title)) {  
			return true;
		    }
		    other_collection = null;
		}
		config_file = null;
	    }
	}
	return false;
    }

    private boolean cancelled;
    private File base_final;
    private JButton create_button;
    private JComboBox base_collection;
    private JDialog self;
    private JTextArea description;
    /* Suppress email
    private JTextField address;
    private JTextField host;
    */
    private JTextField title;
    //private RestrictedTextField file;
    private String description_final;
    // private String email_final; no more email
    // private String name_final;
    private String title_final;
    static private Dimension COMPONENT_SIZE = new Dimension(230, 25);
    /** The size of this new collection dialog box. */
    static private Dimension SIZE = new Dimension(600, 280);
    static private int FILENAME_SIZE = 8;

    /** Constructor.
     * @see org.greenstone.gatherer.util.Utility
     */
    public NewCollectionDetailsPrompt() {
	super(Gatherer.g_man, true);
	this.cancelled = true;
	this.self = this;
	// Setup
	setJMenuBar(new SimpleMenuBar("creatingacollection"));
	setSize(SIZE);
	Dictionary.setText(this, "NewCollectionPrompt.Title");

	// Model building. Build a model of all of the collections in the gsdl collect directory with the appropriate directories.
	Vector base_collection_model = new Vector();
	File gsdl_collection_directory = new File(Utility.getCollectionDir(Gatherer.config.gsdl_path));
	File[] possible_collections = gsdl_collection_directory.listFiles();
	for (int i = 0; possible_collections != null && i < possible_collections.length; i++) {
	    // If the directory has a etc/collect.cfg file then it looks like a collection
	    File collect_cfg_file = new File(possible_collections[i], Utility.CONFIG_DIR);
	    if (collect_cfg_file.exists()) {
		// If the directory has a metadata/ then the collection can be used as a base
		File metadata_directory = new File(possible_collections[i], Utility.META_DIR);
		if (metadata_directory.exists()) {
		    // Add to model
		    BasicCollectionConfiguration collect_cfg = new BasicCollectionConfiguration(collect_cfg_file);
		    Item item = new Item(possible_collections[i], collect_cfg);
		    if (!base_collection_model.contains(item)) {
			base_collection_model.add(item);
		    }
		}
	    }
	}

	// Sort the result.
	Collections.sort(base_collection_model);
	base_collection_model.add(0, new Item(null, Dictionary.get("NewCollectionPrompt.NewCollection")));

	// Creation
	JPanel content_pane = (JPanel) getContentPane();
	content_pane.setOpaque(true);
	JPanel upper_pane = new JPanel();
	JLabel instructions_label = new JLabel();
	Dictionary.setText(instructions_label, "NewCollectionPrompt.Instructions");
	JPanel title_pane = new JPanel();
	JLabel title_label = new JLabel();
	Dictionary.setText(title_label, "CDM.General.Collection_Name");
	title = new JTextField();
	title.setPreferredSize(COMPONENT_SIZE);
	Dictionary.setTooltip(title, "CDM.General.Collection_Name_Tooltip");
	JPanel name_pane = new JPanel();
	JLabel name_label = new JLabel();
	Dictionary.setText(name_label, "NewCollectionPrompt.Collection_Name");
	//JPanel file_pane = new JPanel();
	//file = new RestrictedTextField(new RestrictedTextDocument(FILENAME_SIZE), "", FILENAME_SIZE);
	//Dictionary.setTooltip(file, "NewCollectionPrompt.Collection_Name_Tooltip");
	//JLabel file_label = new JLabel(".col");
       	/* Suppress Email
	JPanel email_pane = new JPanel();
	JLabel email_label = new JLabel();
        Dictionary.setText(email_label, "NewCollectionPrompt.Collection_Email");
	JPanel host_pane = new JPanel();
	JPanel address_pane = new JPanel();
	address = new JTextField();
	Dictionary.setTooltip(address, "CDM.General.Email.Creator_Tooltip");
	JLabel at_label = new JLabel("@");
	host = new JTextField();
	Dictionary.setTooltip(host, "CDM.General.Email.Creator_Tooltip");
	*/

	JPanel center_pane = new JPanel();
	JPanel description_pane = new JPanel();
	JLabel description_label = new JLabel();
	Dictionary.setText(description_label, "NewCollectionPrompt.Collection_Description");
	description = new JTextArea();
	description.setBackground(Gatherer.config.getColor("coloring.editable_background", false));
	description.setForeground(Gatherer.config.getColor("coloring.editable_foreground", false));
	description.setRows(5);
	Dictionary.setTooltip(description, "CDM.General.Collection_Extra_Tooltip");

	JPanel bottom_pane = new JPanel();
	// Base Collection
	JPanel base_collection_pane = new JPanel();
	JLabel base_collection_label = new JLabel();
	Dictionary.setText(base_collection_label, "NewCollectionPrompt.Base_Collection");
	base_collection = new JComboBox(base_collection_model);
	Dictionary.setTooltip(base_collection, "NewCollectionPrompt.Base_Collection_Tooltip");

	JPanel button_pane = new JPanel();
	create_button = new GLIButton();
	create_button.setMnemonic(KeyEvent.VK_O);
	Dictionary.setBoth(create_button, "General.OK", "General.OK_Tooltip");
	JButton cancel_button = new GLIButton();
	cancel_button.setMnemonic(KeyEvent.VK_C);
	Dictionary.setBoth(cancel_button, "General.Cancel", "General.Cancel_Tooltip");

	//ColorListener email_color_listener = new ColorListener(address, host); No more email

	// Connection
	cancel_button.addActionListener(new CancelListener());
	create_button.addActionListener(new CreateListener());
	//address.addKeyListener(email_color_listener); suppress email
	description.addKeyListener(new ColorListener(description));
	description.addKeyListener(new DescriptionListener());
	//file.addKeyListener(new ColorListener(file));
	//host.addKeyListener(email_color_listener); suppress email
	title.addKeyListener(new ColorListener(title));
	//title.getDocument().addDocumentListener(new TitleListener());
	// Layout

	title_pane.setLayout(new BorderLayout(5,0));
	title_pane.add(title_label, BorderLayout.WEST);
	title_pane.add(title, BorderLayout.CENTER);

	/* suppress filename
	file_pane.setLayout(new BorderLayout());
	file_pane.add(file, BorderLayout.WEST);
	file_pane.add(file_label, BorderLayout.CENTER);

	name_pane.setLayout(new BorderLayout());
	name_pane.add(name_label, BorderLayout.WEST);
	name_pane.add(file_pane, BorderLayout.CENTER);
	*/

	/* suppress email

	address_pane.setLayout(new BorderLayout());
	address_pane.add(address, BorderLayout.CENTER);
	address_pane.add(at_label, BorderLayout.EAST);

	host_pane.setLayout(new GridLayout(1,2));
	host_pane.add(address_pane);
	host_pane.add(host);

	email_pane.setLayout(new BorderLayout());
	email_pane.add(email_label, BorderLayout.WEST);
	email_pane.add(host_pane, BorderLayout.CENTER);
	*/

	upper_pane.setLayout(new GridLayout(2,1));
	upper_pane.add(instructions_label);
	upper_pane.add(title_pane);
	//upper_pane.add(name_pane);
	//upper_pane.add(email_pane); no email anymore

	description_pane.setLayout(new BorderLayout());
	description_pane.add(description_label, BorderLayout.NORTH);
	description_pane.add(new JScrollPane(description), BorderLayout.CENTER);

	base_collection_pane.setLayout(new BorderLayout(5,0));
	base_collection_pane.add(base_collection_label, BorderLayout.WEST);
	base_collection_pane.add(base_collection, BorderLayout.CENTER);
	// base_collection_pane.add(base_collection_browse, BorderLayout.EAST);

	center_pane.setBorder(BorderFactory.createEmptyBorder(5,0,5,0));
	center_pane.setLayout(new BorderLayout());
	center_pane.add(description_pane, BorderLayout.CENTER);

	bottom_pane.setLayout(new BorderLayout());
	bottom_pane.add(base_collection_pane, BorderLayout.CENTER);
	bottom_pane.add(button_pane, BorderLayout.SOUTH);

	button_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	button_pane.setLayout(new GridLayout(1,2));
	button_pane.add(create_button);
	button_pane.add(cancel_button);

	content_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	content_pane.setLayout(new BorderLayout());
	content_pane.add(upper_pane, BorderLayout.NORTH);
	content_pane.add(center_pane, BorderLayout.CENTER);
	content_pane.add(bottom_pane, BorderLayout.SOUTH);
	// Final dialog setup & positioning.
	Dimension screen_size = Gatherer.config.screen_size;
	setLocation((screen_size.width - SIZE.width) / 2, (screen_size.height - SIZE.height) / 2);
	setVisible(true);
    }

    public boolean isCancelled() {
	return cancelled;
    }

    public File getBase() {
	return base_final;
    }

    public String getDescription() {
	return description_final;
    }

    /* No more email
    public String getEmail() {
	return email_final;
    }
    */

    /** Generates the collection short filename by taking the first eight characters of the title (spaces removed) and then adjusting by further truncating and then adding an unique suffix as necessary.
     * @return the filename as a String
     */
	public String getName() {
		// Retrieve the first 8 non-whitespace characters of title_final.
		StringBuffer name_buffer = new StringBuffer("");
		int i = 0;
		while(i < title_final.length() && name_buffer.length() < 8) {
			char c = title_final.charAt(i);
			// Arg. We need a few more tests than just whitespace
			if(Character.isLetterOrDigit(c)) {
			    name_buffer.append(Character.toLowerCase(c));
			}
			i++;
		}
		// We need to ensure the filename is unique
		int counter = 0;
		StringBuffer new_name_buffer = new StringBuffer(name_buffer.toString());
		while(filenameClashes(new_name_buffer.toString())) {
			new_name_buffer = new StringBuffer(name_buffer.toString());
			counter++;
			String suffix = String.valueOf(counter);
			// If we have to truncate the namestring so as to fit the suffix
			if(suffix.length() + new_name_buffer.length() > 8) {
				new_name_buffer.replace(new_name_buffer.length() - suffix.length(), new_name_buffer.length(), suffix);
			}
			// Or just append it if that isn't necessary
			else {
				new_name_buffer.append(suffix);
			}

		}
		// All done
		return new_name_buffer.toString();
	}

    private boolean filenameClashes(String filename) {
	File collection_directory = new File(Utility.getCollectionDir(Gatherer.config.gsdl_path));
	File children[] = collection_directory.listFiles();
	for(int i = 0; children != null && i < children.length; i++) {
	    if(children[i].getName().equals(filename)) {
		return true;
	    }
	}
	return false;
    }

    public String getTitle() {
	return title_final;
    }


    private class BrowseListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    File file;
	    if(Gatherer.config.gsdl_path != null) {
		file = new File(Utility.getCollectionDir(Gatherer.config.gsdl_path));
	    }
	    else {
		file = new File(Utility.BASE_DIR);
	    }
	    // Show OpenCollectionPrompt.
	    OpenCollectionDialog chooser = new OpenCollectionDialog(file);
	    file = chooser.getSelectedFile();
	    if(file != null) {
		file = file.getParentFile();
		BasicCollectionConfiguration collect_cfg = new BasicCollectionConfiguration(new File(file, Utility.META_DIR));
		Item item = new Item(file, collect_cfg);
		base_collection.addItem(item);
		base_collection.setSelectedItem(item);
		collect_cfg = null;
	    }
	}
    }

    private class CancelListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    cancelled = true;
	    self.dispose();
	}
    }

    private class ColorListener
	extends KeyAdapter {
	private JTextComponent component1;
	private JTextComponent component2;
	public ColorListener(JTextComponent component) {
	    this.component1 = component;
	    this.component2 = null;
	}
	public ColorListener(JTextComponent component1, JTextComponent component2) {
	    this.component1 = component1;
	    this.component2 = component2;
	}
	public void keyPressed(KeyEvent event) {
	    component1.setForeground(Gatherer.config.getColor("coloring.editable_foreground", false));
	    component1.setBackground(Gatherer.config.getColor("coloring.editable_background", false));
	    if(component2 != null) {
		component2.setForeground(Gatherer.config.getColor("coloring.editable_foreground", false));
		component2.setBackground(Gatherer.config.getColor("coloring.editable_background", false));
	    }
	}
    }

    private class CreateListener
	implements ActionListener {
	
	public void actionPerformed(ActionEvent event) {
	    // Validate.
	    title_final = title.getText();
	    if(title_final.length() == 0) {
		JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("NewCollectionPrompt.Title_Error"), Dictionary.get("NewCollectionPrompt.Error"), JOptionPane.ERROR_MESSAGE);
		title.setForeground(Gatherer.config.getColor("coloring.error_foreground", false));
		title.setBackground(Gatherer.config.getColor("coloring.error_background", false));
		return;
	    }
	    // We must ensure that the collection title is unique. This is a pain in the nether regions as we are forced to load the collect.cfg of each other collection in turn looking for a conflicting title
	    else {
		if(titleClashes(title_final, null)) {
		    if(JOptionPane.showConfirmDialog(Gatherer.g_man, Dictionary.get("NewCollectionPrompt.Title_Clash"), Dictionary.get("General.Warning"), JOptionPane.OK_CANCEL_OPTION) == JOptionPane.NO_OPTION) {
			title.setForeground(Gatherer.config.getColor("coloring.error_foreground", false));
			title.setBackground(Gatherer.config.getColor("coloring.error_background", false));
			return;
		    }
		}
	    }
	    /* Suppress filename
	       name_final = file.getText();
	       if(name_final.length() > 0) {
	       // Determine if this filename is already in use.
	       File collection_directory = new File(Utility.getCollectionDir(Gatherer.config.gsdl_path));
	       File children[] = collection_directory.listFiles();
	       for(int i = 0; children != null && i < children.length; i++) {
	       if(children[i].getName().equals(name_final)) {
	       JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("NewCollectionPrompt.Name_Error"), Dictionary.get("NewCollectionPrompt.Error"), JOptionPane.ERROR_MESSAGE);
	       file.setForeground(Gatherer.config.getColor("coloring.error_foreground", false));
	       file.setBackground(Gatherer.config.getColor("coloring.error_background", false));
	       return;
	       }
	       }
	       }
	       else {
	       JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("NewCollectionPrompt.Name_Error"), Dictionary.get("NewCollectionPrompt.Error"), JOptionPane.ERROR_MESSAGE);
	       file.setForeground(Gatherer.config.getColor("coloring.error_foreground", false));
	       file.setBackground(Gatherer.config.getColor("coloring.error_background", false));
	       return;
	       }
	    */
	    /* Suppress email
	       email_final = address.getText() + "@" + host.getText();
	       if(email_final.length() == 0 || email_final.startsWith("@") || email_final.endsWith("@")) {
	       JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("NewCollectionPrompt.Email_Error"), Dictionary.get("NewCollectionPrompt.Error"), JOptionPane.ERROR_MESSAGE);
	       address.setForeground(Gatherer.config.getColor("coloring.error_foreground", false));
	       address.setBackground(Gatherer.config.getColor("coloring.error_background", false));
	       host.setForeground(Gatherer.config.getColor("coloring.error_foreground", false));
	       host.setBackground(Gatherer.config.getColor("coloring.error_background", false));
	       return;
	       }
	    */
	    description_final = description.getText();
	    if(description_final.length() == 0) {
		JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("NewCollectionPrompt.Description_Error"), Dictionary.get("NewCollectionPrompt.Error"), JOptionPane.ERROR_MESSAGE);
		description.setForeground(Gatherer.config.getColor("coloring.error_foreground", false));
		description.setBackground(Gatherer.config.getColor("coloring.error_background", false));
		return;
	    }
	    // If we got this far there are no errors.
	    Item item_final = (Item) base_collection.getSelectedItem();
	    base_final = item_final.getFile();
	    
	    cancelled = false;
	    
	    self.dispose();
	}
    }
    
    private class DescriptionListener
	extends KeyAdapter {
	public void keyPressed(KeyEvent event) {
	    if(event.getKeyCode() == KeyEvent.VK_TAB) {
		event.consume();
		base_collection.grabFocus();
	    }
	}
    }

    private class Item
	implements Comparable {
	private BasicCollectionConfiguration config;
	private File file;
	private String name;
	public Item(File file, BasicCollectionConfiguration config) {
	    this.config = config;
	    this.file = file;
	    this.name = null;
	}
	public Item(File file, String name) {
	    this.config = null;
	    this.file = file;
	    this.name = name;
	}
	public int compareTo(Object other) {
	    return toString().toLowerCase().compareTo(other.toString().toLowerCase());
	}
	public boolean equals(Object other) {
	    return compareTo(other) == 0;
	}
	public File getFile() {
	    return file;
	}
	public String toString() {
	    if(name == null && config != null) {
		StringBuffer title_buffer = new StringBuffer(config.getName());
		title_buffer.append(StaticStrings.SPACE_CHARACTER);
		title_buffer.append(StaticStrings.OPEN_PARENTHESIS_CHARACTER);
		title_buffer.append(config.getShortName());
		title_buffer.append(StaticStrings.CLOSE_PARENTHESIS_CHARACTER);
		name = title_buffer.toString();
		title_buffer = null;
	    }
	    return name;
	}
    }

    /*
    private class RestrictedTextField
	extends JTextField {
	public RestrictedTextField(RestrictedTextDocument document, String value, int cols) {
	    super(document, "", cols);
	}
	protected Document createDefaultModel() {
	    return new RestrictedTextDocument(11);
	}
    }

    private class RestrictedTextDocument
	extends PlainDocument {
	private char block[];
	private int cols;
	private int current;
	public RestrictedTextDocument(int cols) {
	    super();
	    this.cols = cols;
	    this.current = 0;
	}
	public void blockChar(char c) {
	    if(block == null) {
		block = new char[1];
		block[0] = c;
	    }
	    else {
		char temp[] = block;
		block = new char[temp.length + 1];
		System.arraycopy(temp, 0, block, 0, temp.length);
		block[temp.length] = c;
		temp = null;
	    }
	}
	public void insertString(int offs, String str, AttributeSet a)
	    throws BadLocationException {
	    // Remove any blocked characters.
	    StringBuffer temp = new StringBuffer(str);
	    for(int i = 0; block != null && i < block.length; i++) {
		for(int j = temp.length() - 1; j >= 0; j--) {
		    if(temp.charAt(j) == block[i]) {
			temp.deleteCharAt(j);
		    }
		}
	    }
	    str = temp.toString();
	    if(cols == -1 || str.length() + current <= cols) {
		super.insertString(offs, str, a);
		current = current + str.length();
	    }
	}
	public void remove(int offs, int len)
	    throws BadLocationException {
	    super.remove(offs, len);
	    current = current - len;
	}
    }
    */

    /* Suppress filename
    private class TitleListener
	implements DocumentListener {
	** Gives notification that an attribute or set of attributes changed. *
	public void changedUpdate(DocumentEvent e) {
	    updateFilename();
	}

	** Gives notification that there was an insert into the document. *
	public void insertUpdate(DocumentEvent e) {
	    updateFilename();
	}

	** Gives notification that a portion of the document has been removed. *
	public void removeUpdate(DocumentEvent e) {
	    updateFilename();
	}

	private void updateFilename() {
	    String current_name = file.getText();
	    String current_title = title.getText();
	    StringBuffer temp = new StringBuffer("");
	    int i = 0;
	    while(i < current_title.length() && temp.length() < 8) {
		if(current_title.charAt(i) != ' ') {
		    temp.append(Character.toLowerCase(current_title.charAt(i)));
		}
		i++;
	    }
	    String result = temp.toString();
	    if(current_name.startsWith(result) || result.startsWith(current_name)) {
		file.setText(result);
	    }
	}
    }
    */
}



